#!/usr/bin/env python

# Installation script for Teuthis 1.0
#
# Paul Ricker
# National Center for Supercomputing Applications
# November 2005
#
# Please see the accompanying README file for the license terms associated
# with your use of this software.

import os, os.path, shutil, sys

def set_permissions (arg, dir, names):
    """
    Check all files in a directory and make sure they have safe permissions.
    Python scripts are marked world-executable; all other files are marked
    world-readonly.
    """
    for item in names:
        ext  = os.path.splitext(item)[1]
        path = os.path.join(dir, item)
        if (ext.lower() == ".py") or os.path.isdir(path):
            os.chmod(path, 0755)
        elif os.path.isfile(path):
            os.chmod(path, 0644)

def replace_string (file, pattern, newstr):
    """
    Read a text file and replace all occurrences of pattern with newstr.
    Write the file out again.
    """
    f = open(file, "r")
    indata = f.read()
    f.close()
    outdata = indata.replace(pattern, newstr)
    f = open(file, "w")
    f.write(outdata)
    f.close()

# Configuration

ReleaseVersion = "1.0-beta-1"

cwd    = os.getcwd()
MyPath = os.path.join(cwd, os.path.basename(sys.argv[0]))
argdir = os.path.dirname(sys.argv[0])

if not (os.path.exists(MyPath) and (argdir in ["", ".", cwd])):
    print "This installation script should be run from the directory in"
    print "which it resides."
    sys.exit(1)

# Get information from the user

print "Installing Teuthis version "+ReleaseVersion+".\n"

InstallDirValid = False
while not InstallDirValid:
    InstallDirectory = "/usr/local"
    result = raw_input("Install Teuthis under what directory? [" + \
                       InstallDirectory + "] ")
    if (result != ""):
        InstallDirectory = result
    if os.path.isdir(InstallDirectory):
        if os.access(InstallDirectory, os.W_OK):
            InstallDirValid = True
        else:
            print "\nYou do not have write access to " + \
                  InstallDirectory + ".\n"
    else:
        print '\nPath "'+InstallDirectory+'" does not exist or is not a '+ \
              'directory.\n'

InstallDirectory = os.path.join(InstallDirectory, "teuthis-"+ReleaseVersion)
print "\nWill install to "+InstallDirectory+"."
if os.path.isdir(InstallDirectory):
    print
    RemoveOldInstall = "blah"
    while not (RemoveOldInstall.lower() in ["yes", "no"]):
        RemoveOldInstall = "yes"
        result = raw_input("An existing installation has been found in this "+\
                           "location.  Remove it? ["+RemoveOldInstall+"] ")
        if (result != ""):
            RemoveOldInstall = result
    if (RemoveOldInstall.lower() == "yes"):
        try:
            shutil.rmtree(InstallDirectory)
        except:
            print "\nCould not completely remove the old installation."
            sys.exit(1)

CreateLinks = "blah"
while not (CreateLinks.lower() in ["yes", "no"]):
    print
    CreateLinks = "yes"
    result = raw_input("Create links to executable files? ["+CreateLinks+"] ")
    if (result != ""):
        CreateLinks = result

if (CreateLinks.lower() == "yes"):
    print
    LinksDirValid = False
    while not LinksDirValid:
        LinksDirectory = "/usr/local/bin"
        result = raw_input("Directory in which to create links? ["+ \
                           LinksDirectory+"] ")
        if (result != ""):
            LinksDirectory = result
        if os.path.isdir(LinksDirectory):
            if os.access(LinksDirectory, os.W_OK):
                LinksDirValid = True
            else:
                print "\nYou do not have write access to " + \
                      LinksDirectory + ".\n"
        else:
            print '\nPath "'+LinksDirectory+'" does not exist or is not a '+ \
                  'directory.\n'

# Perform the installation

try:
    print "\nCopying files..."
    shutil.copytree(cwd, InstallDirectory)

    print "\nApplying installation settings..."
    executable = os.path.join(InstallDirectory, "bin", "teuthis.py")
    gladefile  = os.path.join(InstallDirectory, "lib", "teuthis.glade")
    replace_string(executable, "<<<INSTALLATION PATH>>>", InstallDirectory)
    replace_string(gladefile, "<<<INSTALLATION PATH>>>", InstallDirectory)

    print "\nSetting permissions..."
    os.path.walk(InstallDirectory, set_permissions, None)

    if (CreateLinks.lower() == "yes"):
        print "\nCreating links..."
        bindir = os.path.join(InstallDirectory, "bin")
        if os.path.isdir(bindir):
            for file in os.listdir(bindir):
                path = os.path.join(bindir, file)
                base = os.path.splitext(file)[0]
                ext  = os.path.splitext(file)[1]
                link = os.path.join(LinksDirectory, base)
                if (ext.lower() == ".py") and (not os.path.exists(link)):
                    os.symlink(path, link)
except OSError, e:
    print "I/O operation failed.  Error:  %s" % e
    sys.exit(1)

print "\nInstallation complete."

sys.exit(0)
